/* gnome-db-raw-form.c
 *
 * Copyright (C) 2002 - 2006 Vivien Malerba
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <string.h>
#include <libgda/libgda.h>
#include <glib/gi18n-lib.h>
#include "gnome-db-raw-form.h"
#include "gnome-db-data-widget.h"
#include "gnome-db-basic-form.h"

static void gnome_db_raw_form_class_init (GnomeDbRawFormClass * class);
static void gnome_db_raw_form_init (GnomeDbRawForm *wid);
static void gnome_db_raw_form_dispose (GObject   *object);

static void gnome_db_raw_form_set_property (GObject *object,
					    guint param_id,
					    const GValue *value,
					    GParamSpec *pspec);
static void gnome_db_raw_form_get_property (GObject *object,
					    guint param_id,
					    GValue *value,
					    GParamSpec *pspec);

static void gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTable *box_widgets);

static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
static void proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
static void proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm *form);

/* GnomeDbDataWidget interface */
static void            gnome_db_raw_form_widget_init         (GnomeDbDataWidgetIface *iface);
static GdaDataProxy   *gnome_db_raw_form_get_proxy           (GnomeDbDataWidget *iface);
static void            gnome_db_raw_form_col_set_show        (GnomeDbDataWidget *iface, gint column, gboolean shown);
static void            gnome_db_raw_form_set_column_editable (GnomeDbDataWidget *iface, gint column, gboolean editable);
static void            gnome_db_raw_form_show_column_actions (GnomeDbDataWidget *iface, gint column, gboolean show_actions);
static GtkActionGroup *gnome_db_raw_form_get_actions_group   (GnomeDbDataWidget *iface);
static GdaDataModelIter *gnome_db_raw_form_widget_get_data_set   (GnomeDbDataWidget *iface);
static GdaDataModel   *gnome_db_raw_form_widget_get_gda_model (GnomeDbDataWidget *iface);
static void            gnome_db_raw_form_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model);


struct _GnomeDbRawFormPriv
{
	GdaDataModel      *model;
	GdaDataProxy      *proxy; /* proxy for @model */
	GdaDataModelIter  *iter;  /* proxy's iter */

	GtkActionGroup    *actions_group;
};

#define PAGE_NO_DATA 0
#define PAGE_FORM    1

/* get a pointer to the parents to be able to call their destructor */
static GObjectClass *parent_class = NULL;

/* properties */
enum
{
        PROP_0,
	PROP_MODEL,
};

GType
gnome_db_raw_form_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbRawFormClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gnome_db_raw_form_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbRawForm),
			0,
			(GInstanceInitFunc) gnome_db_raw_form_init
		};		
		
		static const GInterfaceInfo work_widget_info = {
                        (GInterfaceInitFunc) gnome_db_raw_form_widget_init,
                        NULL,
                        NULL
                };
		
		type = g_type_register_static (GNOME_DB_TYPE_BASIC_FORM, "GnomeDbRawForm", &info, 0);
		g_type_add_interface_static (type, GNOME_DB_TYPE_DATA_WIDGET, &work_widget_info);
	}

	return type;
}

static void
gnome_db_raw_form_widget_init (GnomeDbDataWidgetIface *iface)
{
	iface->get_proxy = gnome_db_raw_form_get_proxy;
	iface->col_set_show = gnome_db_raw_form_col_set_show;
	iface->set_column_editable = gnome_db_raw_form_set_column_editable;
	iface->show_column_actions = gnome_db_raw_form_show_column_actions;
	iface->get_actions_group = gnome_db_raw_form_get_actions_group;
	iface->get_data_set = gnome_db_raw_form_widget_get_data_set;
	iface->get_gda_model = gnome_db_raw_form_widget_get_gda_model;
	iface->set_gda_model = gnome_db_raw_form_widget_set_gda_model;
}


static void
gnome_db_raw_form_class_init (GnomeDbRawFormClass * class)
{
	GObjectClass   *object_class = G_OBJECT_CLASS (class);
	
	parent_class = g_type_class_peek_parent (class);
	object_class->dispose = gnome_db_raw_form_dispose;

	/* Properties */
        object_class->set_property = gnome_db_raw_form_set_property;
        object_class->get_property = gnome_db_raw_form_get_property;
	g_object_class_install_property (object_class, PROP_MODEL,
					 g_param_spec_object ("model", _("Data to display"), NULL, GDA_TYPE_DATA_MODEL,
                                                               G_PARAM_READABLE | G_PARAM_WRITABLE));
}

static void action_new_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_delete_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_undelete_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_commit_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_reset_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_first_record_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_prev_record_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_next_record_cb (GtkAction *action, GnomeDbRawForm *form);
static void action_last_record_cb (GtkAction *action, GnomeDbRawForm *form);

static GtkActionEntry ui_actions[] = {
	{ "ActionNew", GTK_STOCK_NEW, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
	{ "ActionDelete", GTK_STOCK_DELETE, "_Delete", NULL, "Delete the selected entry", G_CALLBACK (action_delete_cb)},
	{ "ActionUndelete", GTK_STOCK_UNDELETE, "_Undelete", NULL, "Cancels the deletion of the current entry", 
	  G_CALLBACK (action_undelete_cb)},	
	{ "ActionCommit", GTK_STOCK_SAVE, "_Commit", NULL, "Commit the latest changes", G_CALLBACK (action_commit_cb)},
	{ "ActionReset", GTK_STOCK_REFRESH, "_Reset", NULL, "Reset the data", G_CALLBACK (action_reset_cb)},
	{ "ActionFirstRecord", GTK_STOCK_GOTO_FIRST, "_First record", NULL, "Go to first record of records", 
	  G_CALLBACK (action_first_record_cb)},
	{ "ActionLastRecord", GTK_STOCK_GOTO_LAST, "_Last record", NULL, "Go to last record of records", 
	  G_CALLBACK (action_last_record_cb)},
	{ "ActionPrevRecord", GTK_STOCK_GO_BACK, "_Previous record", NULL, "Go to previous record of records", 
	  G_CALLBACK (action_prev_record_cb)},
	{ "ActionNextRecord", GTK_STOCK_GO_FORWARD, "Ne_xt record", NULL, "Go to next record of records",
	  G_CALLBACK (action_next_record_cb)}
};

static void
gnome_db_raw_form_init (GnomeDbRawForm * wid)
{
	wid->priv = g_new0 (GnomeDbRawFormPriv, 1);

	wid->priv->model = NULL;
	wid->priv->proxy = NULL;
	wid->priv->iter = NULL;

	/* action group */
        wid->priv->actions_group = gtk_action_group_new ("Actions");
        gtk_action_group_add_actions (wid->priv->actions_group, ui_actions, G_N_ELEMENTS (ui_actions), wid);
}

/**
 * gnome_db_raw_form_new
 * @model: a #GdaDataModel, or %NULL
 *
 * Creates a new #GnomeDbRawForm widget to display data in @model
 *
 * Returns: the new widget
 */
GtkWidget *
gnome_db_raw_form_new (GdaDataModel *model)
{
	GObject *obj;
	
	obj = g_object_new (GNOME_DB_TYPE_RAW_FORM, "model", model, NULL);

	return GTK_WIDGET (obj);	
}

static void
gnome_db_raw_form_dispose (GObject *object)
{
	GnomeDbRawForm *form;

	g_return_if_fail (object != NULL);
	g_return_if_fail (GNOME_DB_IS_RAW_FORM (object));
	form = GNOME_DB_RAW_FORM (object);

	if (form->priv) {
		/* proxy's iterator */
		if (form->priv->iter) {
			g_signal_handlers_disconnect_by_func (form->priv->iter,
							      G_CALLBACK (iter_row_changed_cb), form);
			g_object_unref (form->priv->iter);
			form->priv->iter = NULL;
		}
		
		/* proxy */
		if (form->priv->proxy) {
			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
							      G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
							      G_CALLBACK (proxy_changed_cb), form);
			g_object_unref (form->priv->proxy);
			form->priv->proxy = NULL;
		}

		/* UI */
		if (form->priv->actions_group) 
			g_object_unref (G_OBJECT (form->priv->actions_group));

		/* the private area itself */
		g_free (form->priv);
		form->priv = NULL;
	}

	/* for the parent class */
	parent_class->dispose (object);
}

static void
gnome_db_raw_form_set_property (GObject *object,
			    guint param_id,
			    const GValue *value,
			    GParamSpec *pspec)
{
	GnomeDbRawForm *form;
	gpointer ptr;

        form = GNOME_DB_RAW_FORM (object);
        if (form->priv) {
                switch (param_id) {
		case PROP_MODEL:
			ptr = GDA_DATA_MODEL(g_value_get_object (value));
			if (ptr)
				g_return_if_fail (GDA_IS_DATA_MODEL (ptr));
			if (form->priv->proxy) {
				/* remove old data model settings */
				g_signal_handlers_disconnect_by_func (form->priv->iter,
								      G_CALLBACK (iter_row_changed_cb), form);
				g_object_unref (G_OBJECT (form->priv->iter));
				form->priv->iter = NULL;
				
				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
								      G_CALLBACK (proxy_row_inserted_or_removed_cb), 
								      form);
				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
								      G_CALLBACK (proxy_changed_cb), form);
				g_object_unref (G_OBJECT (form->priv->proxy));
				form->priv->proxy = NULL;
				form->priv->model = NULL;
			}
			if (ptr) {
				/* handle the data model */
				if (GDA_IS_DATA_PROXY (ptr)) {
					form->priv->proxy = (GdaDataProxy *) ptr;
					g_object_ref (ptr);
				}
				else
					form->priv->proxy = (GdaDataProxy *) gda_data_proxy_new ((GdaDataModel*) ptr);

				form->priv->model = gda_data_proxy_get_proxied_model (form->priv->proxy);

				form->priv->iter = gda_data_model_create_iter ((GdaDataModel *) form->priv->proxy);
				gda_data_model_iter_set_at_row (form->priv->iter, 0);

				g_signal_connect (form->priv->iter, "row_changed",
						  G_CALLBACK (iter_row_changed_cb), form);

				g_signal_connect (G_OBJECT (form->priv->proxy), "row_inserted",
						  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
				g_signal_connect (G_OBJECT (form->priv->proxy), "row_removed",
						  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
				g_signal_connect (G_OBJECT (form->priv->proxy), "changed",
						  G_CALLBACK (proxy_changed_cb), form);

				/* we don't want chuncking */
				g_object_set (object, "paramlist", form->priv->iter, NULL);
				gda_data_proxy_set_sample_size (form->priv->proxy, 0);
				gnome_db_raw_form_initialize (form, NULL, NULL);
			}

			g_signal_emit_by_name (object, "proxy_changed", form->priv->proxy);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
			break;
                }
        }
}

static void
gnome_db_raw_form_get_property (GObject *object,
			   guint param_id,
			   GValue *value,
			   GParamSpec *pspec)
{
	GnomeDbRawForm *form;

        form = GNOME_DB_RAW_FORM (object);
        if (form->priv) {
                switch (param_id) {
		case PROP_MODEL:
			g_value_set_object(value, form->priv->model);					
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
			break;
		}
        }	
}

static void
iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
{
	gnome_db_basic_form_set_current_as_orig (GNOME_DB_BASIC_FORM (form));

	gtk_widget_set_sensitive (GTK_WIDGET (form), (row == -1) ? FALSE : TRUE);
	if (row >= 0) {
		GSList *params;
		guint attributes;
		GdaParameter *param;

		params = ((GdaParameterList *) iter)->parameters;
		while (params) {
			param = (GdaParameter *) params->data;
			attributes = gda_data_proxy_get_value_attributes (form->priv->proxy, row, 
							       gda_data_model_iter_get_column_for_param (iter, param));
			gnome_db_basic_form_entry_set_editable ((GnomeDbBasicForm *) form, 
								param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
			params = params->next;
		}
	}
}

static void
proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form)
{
	/* TO remove ? */
	gtk_widget_set_sensitive (GTK_WIDGET (form), 
				  gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) == 0 ? FALSE : TRUE);
}

static void
proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm *form)
{
	if (gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) != 0)
		if (gda_data_model_iter_get_row (form->priv->iter) == -1)
			gda_data_model_iter_set_at_row (form->priv->iter, row > 0 ? row - 1 : 0);
}

/*
 * Real initialization
 */
static void 
gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTable *box_widgets)
{
	GSList *list;

	/*
	 * handling of the layout
	 */
	if (layout && box_widgets) {
		/* REM: the @box_widgets has keys for query fields, and we need to provide
		   gnome_db_basic_form_new_in_layout() with a hash table which has keys for GdaParameterListNode;
		   this is why we need to use another hash table: 'fbw' */

		TO_IMPLEMENT;
		/* GSList *list; */
/* 		GdaParameter *param; */
/* 		GdaParameterListNode *node; */
/* 		gpointer widget; */

/* 		fbw = g_hash_table_new (NULL, NULL); */
/* 		g_object_get (G_OBJECT (orig_query), "really_all_fields", &list, NULL); */
/* 		while (list) { */
/* 			widget = g_hash_table_lookup (box_widgets, list->data); */
/* 			if (widget) { */
/* 				node = gnome_db_work_core_find_context_node (form->priv->core, GDA_QUERY_FIELD (list->data)); */
/* 				if (node) */
/* 					g_hash_table_insert (fbw, node, widget); */
/* 			} */
/* 			list = g_slist_next (list); */
/* 		} */
	}

	/* 
	 * the form itself 
	 */
	if (!layout && gda_data_proxy_is_read_only (form->priv->proxy))
		gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (form), FALSE);

	list = GDA_PARAMETER_LIST (form->priv->iter)->parameters;
	while (list) {
		GdaParameterListNode *node;

		node = gda_parameter_list_find_node_for_param (GDA_PARAMETER_LIST (form->priv->iter), 
							       GDA_PARAMETER (list->data));
		g_assert (node);
		if (node->hint & GDA_PARAMETER_LIST_PARAM_HIDE)
			gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form),
							GDA_PARAMETER (list->data), FALSE);
		list = g_slist_next (list);
	}

	/* data display update */
	proxy_changed_cb (form->priv->proxy, form);
}

/*
 *
 * Modification buttons (Commit changes, Reset form, New entry, Delete)
 *
 */
static void
action_new_cb (GtkAction *action, GnomeDbRawForm *form)
{
	gint newrow;
	GError *error = NULL;
	GSList *list;

	/* append a row in the proxy */
	newrow = gda_data_model_append_row (GDA_DATA_MODEL (form->priv->proxy), &error);
	if (newrow == -1) {
		g_warning (_("Can't append row to data model: %s"), 
			   error && error->message ? error->message : _("Unknown error"));
		g_error_free (error);
		return;
	}

	g_assert (gda_data_model_iter_set_at_row (form->priv->iter, newrow));

	/* set parameters to their default values */
	list = GDA_PARAMETER_LIST (form->priv->iter)->parameters;
	while (list) {
		GdaParameter *param;
		const GValue *value;
		
		g_object_get (G_OBJECT (list->data), "full_bind", &param, NULL);
		if (! param) {
			value = gda_parameter_list_get_param_default_value ((GdaParameterList *) (form->priv->iter),
									    GDA_PARAMETER (list->data));
			gda_parameter_set_value (GDA_PARAMETER (list->data), value);
		}
		
		list = g_slist_next (list);
	}
	
	gnome_db_basic_form_set_entries_default (GNOME_DB_BASIC_FORM (form));
	gnome_db_basic_form_set_entries_auto_default (GNOME_DB_BASIC_FORM (form), TRUE);
}

static void
action_delete_cb (GtkAction *action, GnomeDbRawForm *form)
{
	gint row;

	row = gda_data_model_iter_get_row (form->priv->iter);
	g_return_if_fail (row >= 0);
	gda_data_proxy_delete (form->priv->proxy, row);
}

static void
action_undelete_cb (GtkAction *action, GnomeDbRawForm *form)
{
	gint row;

	row = gda_data_model_iter_get_row (form->priv->iter);
	g_return_if_fail (row >= 0);
	gda_data_proxy_undelete (form->priv->proxy, row);
}

static void
action_commit_cb (GtkAction *action, GnomeDbRawForm *form)
{
	gda_data_proxy_apply_all_changes (form->priv->proxy, NULL);
}

static void
action_reset_cb (GtkAction *action, GnomeDbRawForm *form)
{
	gda_data_proxy_cancel_all_changes (form->priv->proxy);
	gda_data_model_send_hint (GDA_DATA_MODEL (form->priv->proxy), GDA_DATA_MODEL_HINT_REFRESH, NULL);
}

static void arrow_actions_real_do (GnomeDbRawForm *form, gint movement);
static void
action_first_record_cb (GtkAction *action, GnomeDbRawForm *form)
{
	arrow_actions_real_do (form, -2);
}

static void
action_prev_record_cb (GtkAction *action, GnomeDbRawForm *form)
{
	arrow_actions_real_do (form, -1);
}

static void
action_next_record_cb (GtkAction *action, GnomeDbRawForm *form)
{
	arrow_actions_real_do (form, 1);
}

static void
action_last_record_cb (GtkAction *action, GnomeDbRawForm *form)
{
	arrow_actions_real_do (form, 2);
}


static void
arrow_actions_real_do (GnomeDbRawForm *form, gint movement)
{
	gint row, oldrow;

	row = gda_data_model_iter_get_row (form->priv->iter);
	g_return_if_fail (row >= 0);
	oldrow = row;
	
	/* see if some data have been modified and need to be written to the DBMS */
	/* if ((form->priv->mode & GNOME_DB_ACTION_MODIF_AUTO_COMMIT) && */
/* 	    gda_data_proxy_has_changed (form->priv->proxy)) */
/* 		action_commit_cb (NULL, form); */
	
	/* movement */
	switch (movement) {
	case -2:
		row = 0;
		break;
	case -1:
		if (row > 0)
			row--;
		break;
	case 1:
		if (row < gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) - 1 )
			row++;
		break;
	case 2:
		row = gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) - 1;
		break;
	default:
		g_assert_not_reached ();
	}

	if (oldrow != row)
		gda_data_model_iter_set_at_row (form->priv->iter, row);
}

/*
 * GnomeDbDataWidget interface implementation
 */
static GdaDataProxy *
gnome_db_raw_form_get_proxy (GnomeDbDataWidget *iface)
{
	GnomeDbRawForm *form;

	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), NULL);
	form = GNOME_DB_RAW_FORM (iface);
	g_return_val_if_fail (form->priv, NULL);

	return form->priv->proxy;
}

static void
gnome_db_raw_form_col_set_show (GnomeDbDataWidget *iface, gint column, gboolean shown)
{
	GnomeDbRawForm *form;
	GdaParameter *param;

	g_return_if_fail (GNOME_DB_IS_RAW_FORM (iface));
	form = GNOME_DB_RAW_FORM (iface);
	g_return_if_fail (form->priv);

	param = gda_data_model_iter_get_param_for_column (form->priv->iter, column);
	g_return_if_fail (param);
	gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form), param, shown);
}

void
gnome_db_raw_form_set_column_editable (GnomeDbDataWidget *iface, gint column, gboolean editable)
{
	GnomeDbRawForm *form;

	g_return_if_fail (GNOME_DB_IS_RAW_FORM (iface));
	form = GNOME_DB_RAW_FORM (iface);
	g_return_if_fail (form->priv);

	TO_IMPLEMENT;
	/* What needs to be done:
	 * - create a gnome_db_basic_form_set_entry_editable() function for GnomeDbBasicForm, and call it
	 * - in the GnomeDbDataEntry, add a GDA_VALUE_ATTR_EDITABLE property which defaults to TRUE.
	 * - imtplement the gnome_db_basic_form_set_entry_editable() in the same way as gnome_db_basic_form_set_entries_default()
	 *   by setting that new property
	 * - implement the new property in GnomeDbEntryWrapper and GnomeDbEntryCombo.
	 */
}

static void
gnome_db_raw_form_show_column_actions (GnomeDbDataWidget *iface, gint column, gboolean show_actions)
{
	GnomeDbRawForm *form;
	
	g_return_if_fail (GNOME_DB_IS_RAW_FORM (iface));
	form = GNOME_DB_RAW_FORM (iface);
	g_return_if_fail (form->priv);
	
	/* REM: don't take care of the @column argument */
	gnome_db_basic_form_show_entries_actions (GNOME_DB_BASIC_FORM (form), show_actions);
}

static GtkActionGroup *
gnome_db_raw_form_get_actions_group (GnomeDbDataWidget *iface)
{
	GnomeDbRawForm *form;
	
	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), NULL);
	form = GNOME_DB_RAW_FORM (iface);
	g_return_val_if_fail (form->priv, NULL);

	return form->priv->actions_group;
}

static GdaDataModelIter *
gnome_db_raw_form_widget_get_data_set (GnomeDbDataWidget *iface)
{
	GnomeDbRawForm *form;
	
	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), NULL);
	form = GNOME_DB_RAW_FORM (iface);
	g_return_val_if_fail (form->priv, NULL);

	return form->priv->iter;
}

static GdaDataModel *
gnome_db_raw_form_widget_get_gda_model (GnomeDbDataWidget *iface)
{
	GnomeDbRawForm *form;
	
	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), NULL);
	form = GNOME_DB_RAW_FORM (iface);
	g_return_val_if_fail (form->priv, NULL);

	return form->priv->model;	
}

static void
gnome_db_raw_form_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model)
{
	GnomeDbRawForm *form;
	
	g_return_if_fail (GNOME_DB_IS_RAW_FORM (iface));
	form = GNOME_DB_RAW_FORM (iface);
	g_return_if_fail (form->priv);

	g_object_set (form, "model", model, NULL);
}
